@@ -3,6 +3,7 @@ This module is an extension for xarray to provide rasterio capabilities
 to xarray datasets/dataarrays.
 """
 # pylint: disable=too-many-lines
+import json
 import math
 import warnings
 from collections.abc import Hashable, Iterable
@@ -141,8 +142,10 @@ def _get_nonspatial_coords(
         src_data_array.rio.x_dim,
         src_data_array.rio.y_dim,
         DEFAULT_GRID_MAP,
+        "xc",
+        "yc",
     }:
-        if src_data_array[coord].dims:
+        if src_data_array[coord].ndim == 1:
             coords[coord] = xarray.IndexVariable(
                 src_data_array[coord].dims,
                 src_data_array[coord].values,
@@ -166,9 +169,13 @@ def _make_coords(
 ) -> dict[Hashable, Any]:
     """Generate the coordinates of the new projected `xarray.DataArray`"""
     coords = _get_nonspatial_coords(src_data_array)
-    if force_generate or (
-        src_data_array.rio.x_dim in src_data_array.coords
-        and src_data_array.rio.y_dim in src_data_array.coords
+    if (
+        force_generate
+        or (
+            src_data_array.rio.x_dim in src_data_array.coords
+            and src_data_array.rio.y_dim in src_data_array.coords
+        )
+        or ("xc" in src_data_array.coords and "yc" in src_data_array.coords)
     ):
         new_coords = _generate_spatial_coords(dst_affine, dst_width, dst_height)
         new_coords.update(coords)
@@ -248,6 +255,7 @@ def _order_bounds(
 class XRasterBase:
     """This is the base class for the GIS extensions for xarray"""
 
+    # pylint: disable=too-many-instance-attributes
     def __init__(self, xarray_obj: Union[xarray.DataArray, xarray.Dataset]):
         self._obj: Union[xarray.DataArray, xarray.Dataset] = xarray_obj
 
@@ -283,6 +291,7 @@ class XRasterBase:
         self._height: Optional[int] = None
         self._width: Optional[int] = None
         self._crs: Union[rasterio.crs.CRS, None, Literal[False]] = None
+        self._gcps: Optional[list[GroundControlPoint]] = None
 
     @property
     def crs(self) -> Optional[rasterio.crs.CRS]:
@@ -341,6 +350,7 @@ class XRasterBase:
         obj_copy.rio._width = self._width
         obj_copy.rio._height = self._height
         obj_copy.rio._crs = self._crs
+        obj_copy.rio._gcps = self._gcps
         return obj_copy
 
     def set_crs(
@@ -425,7 +435,7 @@ class XRasterBase:
                 new_attrs.pop("grid_mapping", None)
                 data_obj[var].rio.update_encoding(
                     {"grid_mapping": grid_mapping_name}, inplace=True
-                ).rio.update_attrs(new_attrs, inplace=True).rio.set_spatial_dims(
+                ).rio.set_attrs(new_attrs, inplace=True).rio.set_spatial_dims(
                     x_dim=x_dim, y_dim=y_dim, inplace=True
                 )
         # remove grid_mapping from attributes if it exists
@@ -434,7 +444,7 @@ class XRasterBase:
         new_attrs.pop("grid_mapping", None)
         return data_obj.rio.update_encoding(
             {"grid_mapping": grid_mapping_name}, inplace=True
-        ).rio.update_attrs(new_attrs, inplace=True)
+        ).rio.set_attrs(new_attrs, inplace=True)
 
     def write_crs(
         self,
@@ -1225,11 +1235,19 @@ class XRasterBase:
         )
         data_obj = self._get_obj(inplace=True)
 
-        data_obj = data_obj.rio.write_crs(
-            gcp_crs, grid_mapping_name=grid_mapping_name, inplace=inplace
-        )
+        if gcp_crs:
+            data_obj = data_obj.rio.write_crs(
+                gcp_crs, grid_mapping_name=grid_mapping_name, inplace=inplace
+            )
+        try:
+            grid_map_attrs = data_obj.coords[grid_mapping_name].attrs.copy()
+        except KeyError:
+            data_obj.coords[grid_mapping_name] = xarray.Variable((), 0)
+            grid_map_attrs = data_obj.coords[grid_mapping_name].attrs.copy()
         geojson_gcps = _convert_gcps_to_geojson(gcps)
-        data_obj.coords[grid_mapping_name].attrs["gcps"] = geojson_gcps
+        grid_map_attrs["gcps"] = json.dumps(geojson_gcps)
+        data_obj.coords[grid_mapping_name].rio.set_attrs(grid_map_attrs, inplace=True)
+        self._gcps = list(gcps)
         return data_obj
 
     def get_gcps(self) -> Optional[list[GroundControlPoint]]:
@@ -1243,8 +1261,10 @@ class XRasterBase:
         list of :obj:`rasterio.control.GroundControlPoint` or None
             The Ground Control Points from the dataset or None if not applicable
         """
+        if self._gcps is not None:
+            return self._gcps
         try:
-            geojson_gcps = self._obj.coords[self.grid_mapping].attrs["gcps"]
+            geojson_gcps = json.loads(self._obj.coords[self.grid_mapping].attrs["gcps"])
         except (KeyError, AttributeError):
             return None
 
@@ -1261,8 +1281,8 @@ class XRasterBase:
                 info=gcp["properties"]["info"],
             )
 
-        gcps = [_parse_gcp(gcp) for gcp in geojson_gcps["features"]]
-        return gcps
+        self._gcps = [_parse_gcp(gcp) for gcp in geojson_gcps["features"]]
+        return self._gcps
 
 
 def _convert_gcps_to_geojson(
